/*
 * JBoss, Home of Professional Open Source
 * Copyright 2006, Red Hat Middleware LLC, and individual contributors 
 * as indicated by the @author tags. 
 * See the copyright.txt in the distribution for a
 * full listing of individual contributors. 
 * This copyrighted material is made available to anyone wishing to use,
 * modify, copy, or redistribute it subject to the terms and conditions
 * of the GNU Lesser General Public License, v. 2.1.
 * This program is distributed in the hope that it will be useful, but WITHOUT A 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License,
 * v.2.1 along with this distribution; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA  02110-1301, USA.
 * 
 * (C) 2005-2006,
 * @author JBoss Inc.
 */
/*
 * Copyright (C) 1998, 1999, 2000,
 *
 * Arjuna Solutions Limited,
 * Newcastle upon Tyne,
 * Tyne and Wear,
 * UK.  
 *
 * $Id: Semaphore.java 2342 2006-03-30 13:06:17Z  $
 */

package com.arjuna.ats.txoj.semaphore;

import com.arjuna.ats.txoj.TxOJNames;
import com.arjuna.ats.txoj.common.*;
import com.arjuna.ats.internal.txoj.Implementations;
import com.arjuna.ats.arjuna.gandiva.inventory.Inventory;
import com.arjuna.ats.arjuna.gandiva.ClassName;
import com.arjuna.ats.arjuna.gandiva.ObjectName;
import com.arjuna.ats.arjuna.common.Uid;

import java.io.IOException;

/**
 * Semaphore interface. Binds to interpreter specific implementation.
 *
 * @author Mark Little (mark@arjuna.com)
 * @version $Id: Semaphore.java 2342 2006-03-30 13:06:17Z  $
 * @since JTS 1.0.
 */

public class Semaphore
{

public static final int SM_LOCKED = 0;
public static final int SM_UNLOCKED = 1;
public static final int SM_WOULD_BLOCK = 2;
public static final int SM_ERROR = 3;

    /**
     * Create a semaphore with the default implementation.
     */
    
public Semaphore (String key)
    {
	Object[] param = new Object[1];

	param[0] = key;
	
	Object ptr = Inventory.inventory().createResources(TxOJNames.Implementation_Semaphore_BasicSemaphore(), param);

	if (ptr instanceof SemaphoreImple)
	    _imple = (SemaphoreImple) ptr;
	else
	    _imple = null;
    }
    
public Semaphore (ClassName type)
    {
	Object ptr = Inventory.inventory().createVoid(type);

	if (ptr instanceof SemaphoreImple)
	    _imple = (SemaphoreImple) ptr;
	else
	    _imple = null;
    }

public Semaphore (ObjectName objName)
    {
	_imple = null;
	
	if (objName != null)
	{
	    try
	    {
		ClassName semaphoreType = objName.getClassNameAttribute("SemaphoreType");
		ObjectName semaphoreName = objName.getObjectNameAttribute("SemaphoreName");
		
		Object ptr = Inventory.inventory().createObjectName(semaphoreType, semaphoreName);

		if (ptr instanceof SemaphoreImple)
		    _imple = (SemaphoreImple) ptr;
	    }
	    catch (IOException e)
	    {
	    }
	}
    }    

public int lock ()
    {
	return ((_imple != null) ? _imple.lock() : Semaphore.SM_ERROR);
    }
    
public int unlock ()
    {
	return ((_imple != null) ? _imple.unlock() : Semaphore.SM_ERROR);
    }
    
public int tryLock ()
    {
	return ((_imple != null) ? _imple.tryLock() : Semaphore.SM_ERROR);
    }
    
public ClassName className ()
    {
	return ((_imple != null) ? _imple.className() : ClassName.invalid());
    }
    
private SemaphoreImple _imple;

    static 
    {
	if (!Implementations.added())
	    Implementations.initialise();
    }
    
}


